import Command from "./command";

import db from "../things/db";

import {
  CommandInteraction,
  Constants,
  InteractionDataOptionsNumber,
  InteractionDataOptionsSubCommand
} from "@projectdysnomia/dysnomia";

import {
  Optional,
  unravelOption,
  unravelOptionalOption
} from "../utils/options";

async function listReminders(interaction: CommandInteraction) {
  const userReminders = await db.reminder.findMany({
    where: {
      user: (interaction.member || interaction.user)!.id
    }
  });

  await interaction.createMessage({
    embeds: [
      {
        title: "Your reminders",
        description: userReminders
          .map((x) => {
            const timestamp = `<t:${Math.floor(x.sendAt.getTime() / 1000)}:R>`;
            const id = `ID: \`${x.id}\``;
            const message = x.message || "No message set";

            return `${timestamp} - ${id} - ${message}`;
          })
          .join("\n")
      }
    ],
    flags: Constants.MessageFlags.EPHEMERAL
  });
}

async function deleteReminder(interaction: CommandInteraction, id: number) {
  const reminder = await db.reminder.findUnique({
    where: {
      id
    }
  });

  if (!reminder) {
    await interaction.createMessage({
      content: "That reminder doesn't exist.",
      flags: Constants.MessageFlags.EPHEMERAL
    });
    return;
  }

  const user = (interaction.member || interaction.user)!.id;
  if (reminder.user !== user) {
    await interaction.createMessage({
      content: "You can't delete someone else's reminder.",
      flags: Constants.MessageFlags.EPHEMERAL
    });
    return;
  }

  await db.reminder.delete({
    where: {
      id
    }
  });

  await interaction.createMessage({
    content: "Reminder deleted.",
    flags: Constants.MessageFlags.EPHEMERAL
  });
}

const reminders: Command = {
  name: "reminders",
  description: "List your set reminders.",
  options: [
    {
      name: "list",
      description: "List all your reminders.",
      type: Constants.ApplicationCommandOptionTypes.SUB_COMMAND
    },
    {
      name: "delete",
      description: "Delete a reminder.",
      type: Constants.ApplicationCommandOptionTypes.SUB_COMMAND,
      options: [
        {
          name: "id",
          description: "The ID of the reminder to delete.",
          required: true,
          type: Constants.ApplicationCommandOptionTypes.NUMBER
        }
      ]
    }
  ],
  command: async (interaction) => {
    const options = interaction.data.options!;
    const listOption = unravelOptionalOption<
      Optional<InteractionDataOptionsSubCommand>
    >("list", options);
    const deleteOption = unravelOptionalOption<
      Optional<InteractionDataOptionsSubCommand>
    >("delete", options);

    if (listOption !== undefined) await listReminders(interaction);
    if (deleteOption !== undefined) {
      const id = unravelOption<InteractionDataOptionsNumber>(
        "id",
        deleteOption.options!
      );

      await deleteReminder(interaction, id.value);
    }
  }
};

export default reminders;
